# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimString,
    HprimList,
    HprimMultiple,
    Segment,
)

logger = logging.getLogger("driverng.pentra_c200")


class PentraC200SegmentH(HeaderSegment):
    sender_name = HprimString(5)
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)
    manufacturer = HprimForeign("M", "header")
    scientific = HprimForeign("S", "header")


class PentraC200SegmentP(Segment):
    patient_id = HprimString(3)
    last_name = HprimString((6, 1), max_length=12)
    first_name = HprimString((6, 2), max_length=12)
    middle_name = HprimString((6, 3), max_length=12)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    physician_id = HprimString(14)
    ssn = HprimString(15)

    orders = HprimForeign("O", "patient")


class PentraC200SegmentO(Segment):
    sample_id = HprimString(3)
    tests = HprimList(5, basefield=HprimMultiple(0, code=HprimString(4)))
    specimen_type = HprimString(16)

    results = HprimForeign("R", "order")


class PentraC200SegmentR(Segment):
    test_id = HprimString((3, 4))
    value = HprimString(4)
    units = HprimString(5)
    flags = HprimString(7)
    complete_date = HprimDate(13)


class PentraC200SegmentC(Segment):
    source = HprimString(3)
    text = HprimString(4)
    type = HprimString(5)


class PentraC200SegmentQ(Segment):
    specimen_id = HprimString(3)
    request_status = HprimString(13)


class PentraC200SegmentL(Segment):
    pass


class PentraC200SegmentS(Segment):
    pass


class PentraC200SegmentM(Segment):
    pass


class Driver(AstmMessageV2):
    SPEC = {"code": "pentra_c200", "label": "Pentra C200"}
    SEGMENT_CLASS_MAP = {
        "H": PentraC200SegmentH,
        "P": PentraC200SegmentP,
        "O": PentraC200SegmentO,
        "R": PentraC200SegmentR,
        "C": PentraC200SegmentC,
        "Q": PentraC200SegmentQ,
        "L": PentraC200SegmentL,
        "S": PentraC200SegmentS,
        "M": PentraC200SegmentM,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    ENCODING = "UTF-8"

    OPTIONS = [
        AstmMessageV2.Option(
            "pentra_c200_sender_name",
            label="Paramétrer le nom de l'émetteur du segment H",
            default="MCA",
        ),
        AstmMessageV2.Option("pentra_c200_qc_prefix", label="Préfixe QC"),
    ]

    def __init__(self, **kwargs):
        self.pentra_c200_sender_name = kwargs.pop("pentra_c200_sender_name", "MCA")
        self.pentra_c200_qc_prefix = kwargs.pop("pentra_c200_qc_prefix", "")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = PentraC200SegmentH(type_segment="H", **kwargs)
        message.end = PentraC200SegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if not context["query"]:
            logger.debug("Only context query accepted")
            return None

        msg = self.create(date=context["now"], sender_name=self.pentra_c200_sender_name)
        for seg_q in context["query"].queries:
            for patient in worklist.get_astm_worklist():
                physician = [
                    p for p in patient.physicians if p.type.lower() == "prescriber"
                ]
                seg_p = msg.patients.add(
                    patient_id=patient.patient_id,
                    last_name=patient.last_name,
                    first_name=patient.first_name,
                    birthdate=patient.birthdate,
                    sex=patient.sex,
                    physician_id=physician[0].code if physician else "",
                )
                for (sample,) in patient.samples:
                    seg_o = seg_p.orders.add(
                        sample_id=seg_q.specimen_id, specimen_type=sample.specimen
                    )
                    for order in sample.orders:
                        t = seg_o.tests.add()
                        t.code = order.code
        if msg.patients:
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(type="patient", sample_id=seg_q.specimen_id)
            return worklist

        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                sample_id = seg_o.sample_id
                comments = []
                if seg_o.commentaires:
                    for seg_c in seg_o.commentaires:
                        comments.append(seg_c.text)
                for seg_r in seg_o.results:
                    # QC
                    if self.pentra_c200_qc_prefix and sample_id.startswith(
                        self.pentra_c200_qc_prefix
                    ):
                        sample = worklist.get_or_create(
                            sample_id=sample_id, control_lot=sample_id, type="control"
                        )
                    # Patient
                    else:
                        sample = worklist.get_or_create(
                            sample_id=sample_id, type="patient"
                        )
                    unit = ""
                    flags = []
                    if seg_r.units:
                        unit = seg_r.units
                    if seg_r.flags:
                        flags.append(seg_r.flags)
                    if seg_r.commentaires:
                        for seg_c in seg_r.commentaires:
                            comments.append(seg_c.text)
                    sample.results.add(
                        code=seg_r.test_id,
                        value=seg_r.value,
                        date=seg_r.complete_date,
                        unit=unit,
                        flags=flags,
                        comments=comments,
                    )
        return worklist
